/* Copyright (C)  2006-2009, Vlad Volodin <vest.84@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _BACKGROUND_H
#define	_BACKGROUND_H

#include <glibmm/ustring.h>
#include <iostream>
#include <exception>
#include <map>

typedef std::map<Glib::ustring, Glib::ustring> TStringMap;

class file_exception : public std::exception
{
public:

    file_exception (std::string _msg, std::string _desc) throw () : msg (_msg), desc (_desc) { }

    virtual ~file_exception () throw () { }

    virtual const char* what () const throw ()
    {
        std::string error = msg + std::string (": ") + desc;
        return error.c_str ();
    }

private:
    std::string msg; // error message
    std::string desc; // description
};

class Background
{
public:
    static const char EXTENSION[];
    static const char BACKGROUND[];

    virtual ~Background () { }

    static void Clear ();
    static void ScanGameDir ();
    static void ScanHomeDir ();
    static TStringMap GetMap ();

    static Glib::ustring GetXDGBaseDir (Glib::ustring env, Glib::ustring def)
    throw (file_exception);

protected:
    static TStringMap bgMap;

    static void ScanDir (const char* path) throw (file_exception);
    static Glib::ustring GetTitle (Glib::ustring fileName);

private:

    Background () { }

    Background (const Background& orig) { }
};

#endif	/* _BACKGROUND_H */

