/* Copyright (C)  2006-2009, Vlad Volodin <vest.84@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "options.h"

#ifndef _BOARDPANEL_H
#define	_BOARDPANEL_H

#include <gtkmm.h>
#include <librsvg/rsvg.h>
#include <librsvg/rsvg-cairo.h>

#include "board.h"

class BoardPanel : public Gtk::DrawingArea
{
public:
    static constexpr double dx = 1.0;
    static constexpr double dy = 1.0;
    static const Glib::ustring NO_BACKGROUND;

    BoardPanel (const Board* board);
    virtual void Assign (const Board* board);
    virtual ~BoardPanel ();

    TCoordCell PositionToCell (double x, double y) const;
    virtual void RedrawCell (const TCoordCell& cell);
    virtual void DrawSquare (int row1, int col1,
                             int row2, int col2,
                             int row3, int col3,
                             int row4, int col4,
                             TPlayer player);

    virtual void AssignBackground (Glib::ustring filePath);

    void SetOptions(const TOptions& options); // set some drawing options

protected:
    //Override default signal handler:
    virtual bool on_expose_event (GdkEventExpose* event);
    virtual void on_size_allocate (Gtk::Allocation& allocation);

    virtual void MakeImage ();
    virtual void RestoreImage (Cairo::RefPtr<Cairo::Context> cr,
                               double x, double y, double width, double height);


private:
    static const int MIN_HEIGHT = 300;

    const Board* board;

    RsvgHandle* rsvgRed;
    RsvgHandle* rsvgBlue;
    RsvgHandle* rsvgWhite;
    RsvgHandle* rsvgEmpty;
    RsvgHandle* rsvgBackground;

    RsvgDimensionData dRed;
    RsvgDimensionData dBlue;
    RsvgDimensionData dWhite;
    RsvgDimensionData dEmpty;
    RsvgDimensionData dBackground;

    // Scale board
    double sx;
    double sy;

    // Scale background
    double sbx;
    double sby;

    double lineWidth;

    bool mustRedraw;
    bool drawBackground;

    Cairo::RefPtr<Cairo::ImageSurface> imageSurface;
    Cairo::RefPtr<Cairo::Context> crMemory;

    Cairo::RefPtr<Cairo::ImageSurface> backgroundSurface;
    Cairo::RefPtr<Cairo::Context> crbgMemory;

    bool hasSquare;
    int row1, col1;
    int row2, col2;
    int row3, col3;
    int row4, col4;
    TPlayer player;
};

#endif	/* _BOARDPANEL_H */

