/* Copyright (C)  2006-2009, Vlad Volodin <vest.84@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _MAINWINDOW_H
#define	_MAINWINDOW_H

#include <gtkmm.h>
//#include <gtkmm/menu.h>
#include "player.h"
#include "board.h"
#include "aieasy.h"
#include "aihard.h"
#include "playerpanel.h"
#include "boardpanel.h"
#include "types.h"
#include "options.h"
#include "statistics.h"

class MainWindow : public Gtk::Window
{
public:
    MainWindow ();
    virtual ~MainWindow ();

protected:
    virtual bool on_board_button_press_event (GdkEventButton* event);
    virtual bool on_timeout ();

private:
    static const int DEFAULT_WIDTH = 480;
    static const int DEFAULT_HEIGHT = 400;

    Board* board;
    Player* playerBlue;
    Player* playerRed;

    Player* currentPlayer;
    TGameState currentState;

    AIBase* blueAI;
    AIBase* redAI;
    AIBase* playerAI;

    sigc::connection aiConnection;
    bool allowInput;

    TOptions options;
    TOptions newOptions;

    TStatistics statistics;

    Gtk::VBox m_mainBox;
    Gtk::HBox m_gameBox;
    Gtk::VBox m_rightPlayerBox;
    Gtk::VBox m_bluePlayerBox;
    Gtk::VBox m_redPlayerBox;
    Gtk::HBox m_statusBox;

    Gtk::Label m_blueLabel;
    Gtk::Label m_redLabel;

    Gtk::Statusbar m_bluePlayerStatusBar;
    Gtk::Statusbar m_redPlayerStatusBar;

    PlayerPanel m_bluePanel;
    PlayerPanel m_redPanel;

    BoardPanel m_boardPanel;

    Gtk::AspectFrame m_boardFrame;

    Glib::RefPtr<Gtk::UIManager> m_refUIManager;
    Glib::RefPtr<Gtk::UIManager> m_refUIBgManager;
    Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
    Glib::RefPtr<Gtk::ActionGroup> m_refActionBgGroup;
    Glib::RefPtr<Gtk::ToggleAction> m_refPanelToggle;

    Gtk::Widget* BuildMenu ();
    void BuildBgSubmenu ();
    virtual void on_menu_game_new ();
    virtual void on_menu_game_stat ();
    virtual void on_menu_game_quit ();
    virtual void on_menu_edit_options ();
    virtual void on_menu_edit_show ();
    virtual void on_menu_edit_background (Glib::ustring filePath);
    virtual void on_menu_help_about ();

    virtual void NewGame ();
    virtual void EndGame ();
    virtual void CheckWinner ();

    virtual void StartAI ();

    virtual void UpdatePlayerStatus ();
};

#endif	/* _MAINWINDOW_H */
