/* Copyright (C)  2006-2009, Vlad Volodin <vest.84@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glibmm/i18n.h>
#include <sys/stat.h>
#include <errno.h>
#include <libxml++-2.6/libxml++/nodes/contentnode.h>

#include <libxml++/libxml++.h>

#ifdef NEED_PROPER_HOME_DIR
#include <unistd.h>
#include <pwd.h>
#endif //NEED_PROPER_HOME_DIR

#include "background.h"
#include "optionswindow.h"
#include "options.h"

OptionsWindow::OptionsWindow (Gtk::Window& parent, const TOptions& _options) :
Gtk::Dialog (_ ("Options"), parent, true, true),
options (_options)
{
    set_border_width (5);

    add_button (Gtk::Stock::OK, Gtk::RESPONSE_OK);
    add_button (Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);

    m_refSizeGroup = Gtk::SizeGroup::create (Gtk::SIZE_GROUP_HORIZONTAL);

    m_optionsTable.set_border_width (5);
    m_optionsTable.set_row_spacings (5);
    m_optionsTable.set_col_spacings (10);

    get_vbox ()->pack_start (m_optionsTable, Gtk::PACK_SHRINK);

    type_listStrings firstPlayer;
    firstPlayer.push_back (_ ("Blue"));
    firstPlayer.push_back (_ ("Red"));

    type_listStrings playerOption;
    playerOption.push_back (_ ("Human"));
    playerOption.push_back (_ ("Easy Computer"));
    playerOption.push_back (_ ("Hard Computer"));

    pm_firstCombo = create_combobox (firstPlayer);
    add_row (m_optionsTable, 0, m_refSizeGroup, _ ("_First player:"),
             *dynamic_cast<Gtk::Widget*> (pm_firstCombo));
    pm_blueCombo = create_combobox (playerOption);
    add_row (m_optionsTable, 1, m_refSizeGroup, _ ("_Blue player:"),
             *dynamic_cast<Gtk::Widget*> (pm_blueCombo));
    pm_redCombo = create_combobox (playerOption);
    add_row (m_optionsTable, 2, m_refSizeGroup, _ ("_Red player:"),
             *dynamic_cast<Gtk::Widget*> (pm_redCombo));

    SetOptions (options);

    pm_firstCombo->signal_changed ().connect (sigc::mem_fun (*this, &OptionsWindow::on_combo_changed));
    pm_blueCombo->signal_changed ().connect (sigc::mem_fun (*this, &OptionsWindow::on_combo_changed));
    pm_redCombo->signal_changed ().connect (sigc::mem_fun (*this, &OptionsWindow::on_combo_changed));

    this->set_tooltip_markup (_ ("Restart game please, for options to take an effect"));
    this->set_has_tooltip (false);

    show_all_children ();
}

OptionsWindow::~OptionsWindow () { }

void OptionsWindow::SetOptions (const TOptions& options)
{
    pm_firstCombo->set_active (static_cast<int> (options.firstPlayer));
    pm_blueCombo->set_active (static_cast<int> (options.bluePlayer));
    pm_redCombo->set_active (static_cast<int> (options.redPlayer));
}

void OptionsWindow::GetOptions (TOptions& options) const
{
    options.firstPlayer = static_cast<TPlayer> (pm_firstCombo->get_active_row_number ());
    options.bluePlayer = static_cast<TPlayerOption> (pm_blueCombo->get_active_row_number ());
    options.redPlayer = static_cast<TPlayerOption> (pm_redCombo->get_active_row_number ());
}

/* Convenience function to create an option menu holding a number of strings */
Gtk::ComboBoxText* OptionsWindow::create_combobox (const std::list<Glib::ustring>& strings)
{
    Gtk::ComboBoxText* pCombo = Gtk::manage (new Gtk::ComboBoxText ());

    for (type_listStrings::const_iterator iter = strings.begin (); iter != strings.end (); iter++)
    {
        pCombo->append_text (*iter);
    }

    return pCombo;
}

// From "Size groups" example (gtkmm-demo), slightly modified for all widgets

void OptionsWindow::add_row (Gtk::Table& table, int row,
                             const Glib::RefPtr<Gtk::SizeGroup>& size_group,
                             const Glib::ustring& label_text,
                             Gtk::Widget& widget)
{
    Gtk::Label* pLabel = Gtk::manage (new Gtk::Label (label_text, true));
    pLabel->set_alignment (Gtk::ALIGN_LEFT, Gtk::ALIGN_BOTTOM);

    table.attach (*pLabel, 0, 1, row, row + 1, Gtk::EXPAND | Gtk::FILL, Gtk::AttachOptions (0));

    pLabel->set_mnemonic_widget (widget);
    size_group->add_widget (widget);

    table.attach (widget, 1, 2, row, row + 1, Gtk::AttachOptions (0), Gtk::AttachOptions (0));
}

// Display a tip to restart a game after options changed

void OptionsWindow::on_combo_changed ()
{
    if (options.firstPlayer != static_cast<TPlayer> (pm_firstCombo->get_active_row_number ()) ||
            options.bluePlayer != static_cast<TPlayerOption> (pm_blueCombo->get_active_row_number ()) ||
            options.redPlayer != static_cast<TPlayerOption> (pm_redCombo->get_active_row_number ()))
        this->set_has_tooltip (true);
    else
        this->set_has_tooltip (false);
}
