/* Copyright (C)  2006-2009, Vlad Volodin <vest.84@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "playerpanel.h"

#include <iostream>
#include <glib-2.0/glib.h>
#include <glibmm/i18n.h>

PlayerPanel::PlayerPanel (const Player* _player) : player (_player)
{
    set_size_request (MIN_WIDTH, MIN_HEIGHT);

    GError* error = NULL;

    rsvgWhite = rsvg_handle_new_from_file (GNOME_PIXMAPSDIR "/white.svg", &error);

    if (player->getPlayer () == BLUE_PLAYER)
        rsvgColor = rsvg_handle_new_from_file (GNOME_PIXMAPSDIR "/blue.svg", &error);
    else if (player->getPlayer () == RED_PLAYER)
        rsvgColor = rsvg_handle_new_from_file (GNOME_PIXMAPSDIR "/red.svg", &error);
    else
        g_message (_ ("Unpredictable player"));
    g_assert ((rsvgWhite != NULL) && (rsvgColor != NULL));

    rsvg_handle_get_dimensions (rsvgColor, &dColor);
    rsvg_handle_get_dimensions (rsvgWhite, &dWhite);
}

PlayerPanel::~PlayerPanel ()
{
    GError* error = NULL;

    g_object_unref (rsvgWhite);
    g_object_unref (rsvgColor);
}

void PlayerPanel::Assign (const Player* _player)
{
    player = _player;
}

bool PlayerPanel::on_expose_event (GdkEventExpose* event)
{
    // This is where we draw on the window
    Glib::RefPtr<Gdk::Window> window = get_window ();
    if (window)
    {
        Gtk::Allocation allocation = get_allocation ();
        const int width = allocation.get_width ();
        const int height = allocation.get_height ();

        Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context ();

        cr->save ();
        // clip to the area indicated by the expose event so that we only redraw
        // the portion of the window that needs to be redrawn
        cr->rectangle (event->area.x, event->area.y,
                       event->area.width, event->area.height);
        cr->clip ();

        gchar colorLeft[3];
        gchar whiteLeft[3];

        g_snprintf (colorLeft, 3, "%d", player->getAttackLeft ());
        g_snprintf (whiteLeft, 3, "%d", player->getBlockLeft ());

        cr->save ();
        cr->scale (2.0, 2.0);

        cr->set_source_rgb (0.1, 0.1, 0.1);
        cr->select_font_face ("Sans",
                              Cairo::FONT_SLANT_NORMAL,
                              Cairo::FONT_WEIGHT_BOLD);
        cr->set_font_size (dColor.height / 2);

        rsvg_handle_render_cairo (rsvgColor, cr->cobj ());

        // Need for calculating colorLeft width and right aligment it
        Cairo::TextExtents extents;
        if (player->getAttackLeft () >= 10)
            cr->get_text_extents ("20", extents);
        else
            cr->get_text_extents (colorLeft, extents);

        //cr->move_to (dColor.width + 5, dColor.height * 3 / 4);
        cr->move_to ((width / 2 - extents.width + dColor.width) / 2, dColor.height * 3 / 4);
        cr->show_text (colorLeft);

        cr->translate (0, dColor.height + 2.5);
        rsvg_handle_render_cairo (rsvgWhite, cr->cobj ());

        cr->get_text_extents ("6", extents);

        //cr->move_to (dWhite.width + 5, dWhite.height * 3 / 4);
        cr->move_to ((width / 2 - extents.width + dColor.width) / 2, dWhite.height * 3 / 4);
        cr->show_text (whiteLeft);

        cr->restore ();

        return true;
    }

    return false;
}

