/* Copyright (C)  2006-2009, Vlad Volodin <vest.84@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glibmm/i18n.h>
#include <list>
#include <gtkmm-2.4/gtkmm/label.h>
#include <gtkmm-2.4/gtkmm/widget.h>
#include <gtkmm-2.4/gtkmm/container.h>
#include <gdkmm-2.4/gdkmm/pixbuf.h>
#include <gtkmm-2.4/gtkmm/box.h>
#include <iostream>

#include "statwindow.h"

const Glib::ustring StatWindow::TABLE[][2] = {
    {"Human", "Human"},
    {"Human", "Easy"},
    {"Human", "Hard"},
    {"Easy", "Easy"},
    {"Easy", "Hard"},
    {"Hard", "Hard"}
};

const int StatWindow::DEFAULT_ICON_SIZE = 36;

StatWindow::StatWindow (Gtk::Window& parent, const TStatistics& _stat) :
Gtk::Dialog (_ ("Quod Statistics"), parent, true, true),
m_statTable (10, 7),
m_blueFirstLabel (_ ("Blue First")),
m_redFirstLabel (_ ("Red First")),
m_blue1Label (_ ("Blue")),
m_blue2Label (_ ("Blue")),
m_red1Label (_ ("Red")),
m_red2Label (_ ("Red"))
{
    set_default_size (400, 200);
    set_border_width (5);

    Glib::RefPtr<Gtk::IconTheme> iconTheme = Gtk::IconTheme::get_default ();
    /*
    Gtk::Image m_Image_Progressive(Gdk::Pixbuf::create_from_file ("/usr/share/icons/gnome/48x48/stock/data/stock_lock.png"));
    get_vbox ()->pack_start (m_Image_Progressive, Gtk::PACK_SHRINK);
     */
    //iconTheme = Gtk::IconTheme::get_default ();
    //std::cout << (.c_str) << std::endl;
    //std::cout << () << std::endl;
    //     Glib::ustring str = "123123";
    //     Glib::ArrayHandle<Glib::ustring> p = icon->get_search_path ();
    //     for (Glib::ArrayHandle<Glib::ustring>::iterator i = p.begin (); i != p.end (); i++)
    //         std::cout << *i << std::endl;
    /*
        Glib::ListHandle<Glib::ustring> list = icon->list_icons ();
        for (Glib::ListHandle<Glib::ustring>::iterator i = list.begin (); i != list.end (); i++)
        {
            Glib::ustring s = *i;

            if (s.find ("stock_pe") != Glib::ustring::npos)
                std::cout << s << std::endl;
        }
     */
    //static Gtk::Image m_Image_Progressive (icon->load_icon ("stock_people", 24));
    //static Gtk::Image m_Image_Progressive  ("/usr/share/icons/gnome/48x48/stock/data/stock_lock.png");
    //std::cout << m_Image_Progressive.get_storage_type () << std::endl;
    //get_vbox ()->pack_start (m_Image_Progressive, Gtk::PACK_SHRINK);

    m_statTable.attach (m_vertSeparator1, 1, 2, 0, 10, Gtk::FILL);
    m_statTable.attach (m_vertSeparator2, 4, 5, 0, 10, Gtk::FILL);

    m_statTable.attach (m_horSeparator1, 1, 7, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::EXPAND);
    m_statTable.attach (m_horSeparator2, 0, 7, 3, 4, Gtk::FILL | Gtk::EXPAND, Gtk::EXPAND);

    m_statTable.attach (m_blueFirstLabel, 2, 4, 0, 1, Gtk::EXPAND);
    m_statTable.attach (m_redFirstLabel, 5, 7, 0, 1, Gtk::EXPAND);

    m_statTable.attach (*StatWindow::get_image (iconTheme, "Blue"), 2, 3, 2, 3, Gtk::EXPAND);
    m_statTable.attach (*StatWindow::get_image (iconTheme, "Red"), 3, 4, 2, 3, Gtk::EXPAND);
    m_statTable.attach (*StatWindow::get_image (iconTheme, "Blue"), 5, 6, 2, 3, Gtk::EXPAND);
    m_statTable.attach (*StatWindow::get_image (iconTheme, "Red"), 6, 7, 2, 3, Gtk::EXPAND);


    for (int i = 0; i < 6; i++)
    {
        Gtk::HBox* box = Gtk::manage (new Gtk::HBox (true, 5));
        //Gtk::Label* l = Gtk::manage (new Gtk::Label);
        //l->set_text (TABLE[i][0]);
        //Gtk::Label* r = Gtk::manage (new Gtk::Label);
        //r->set_text (TABLE[i][1]);
        box->set_spacing (10);
        box->set_border_width (5);

        box->pack_start (*StatWindow::get_image (iconTheme, TABLE[i][0]));
        box->pack_start (*StatWindow::get_image (iconTheme, TABLE[i][1]));

        m_statTable.attach (*box, 0, 1, i + 4, i + 5, Gtk::FILL);
    }

    // Fill stat numbers into table
    const TPlayerStat * plStat[] = {&_stat.blue_blue, &_stat.blue_red,
        &_stat.red_blue, &_stat.red_red};

    for (int i = 0; i < 4; i++)
    {
        const int SCORES[] = {plStat[i]->human_human, plStat[i]->human_easy,
            plStat[i]->human_hard, plStat[i]->easy_easy,
            plStat[i]->easy_hard, plStat[i]->hard_hard};
        for (int j = 0; j < 6; j++)
        {
            m_statTable.attach (*Gtk::manage (new Gtk::Label (Glib::ustring::format (SCORES[j]))),
                                i + 2 + (i > 1 ? 1 : 0), i + 3 + (i > 1 ? 1 : 0),
                                4 + j, 5 + j, Gtk::FILL);
        }
    }

    get_vbox ()->pack_start (m_statTable, Gtk::PACK_SHRINK);

    add_button (Gtk::Stock::OK, Gtk::RESPONSE_OK);

    show_all_children ();
}

StatWindow::~StatWindow () { }

Gtk::Widget* StatWindow::get_image (Glib::RefPtr<Gtk::IconTheme> iconTheme,
                                    Glib::ustring name)
{
    Gtk::Image* image;
    Glib::ustring tooltip;

    Glib::RefPtr<Gdk::Pixbuf> pixbuf;
    if (name == "Human")
    {
        pixbuf = iconTheme->load_icon ("stock_person", DEFAULT_ICON_SIZE);
        tooltip = _ ("Human player");
    }
    else if (name == "Easy")
    {
        pixbuf = iconTheme->load_icon ("weather-few-clouds", DEFAULT_ICON_SIZE);
        tooltip = _ ("Easy-level computer player");
    }
    else if (name == "Hard")
    {
        pixbuf = iconTheme->load_icon ("weather-storm", DEFAULT_ICON_SIZE);
        tooltip = _ ("Hard-level computer player");
    }
    else if (name == "Red")
        pixbuf = Gdk::Pixbuf::create_from_file (GNOME_PIXMAPSDIR "/red.svg", DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
    else if (name == "Blue")
        pixbuf = Gdk::Pixbuf::create_from_file (GNOME_PIXMAPSDIR "/blue.svg", DEFAULT_ICON_SIZE, DEFAULT_ICON_SIZE);
    //    else
    //        pixBuf = iconTheme->load_icon ("weather-storm", 24);

    image = Gtk::manage (new Gtk::Image (pixbuf));
    if (!tooltip.empty ())
        image->set_tooltip_markup (tooltip);

    return dynamic_cast<Gtk::Widget*> (image);
}
